import { FieldValidationConfiguration, FormDefinition, StudioComponent, StudioComponentChild, StudioForm, FieldConfigMetadata, FormMetadata } from '@aws-amplify/codegen-ui';
import { BindingElement, Expression, ObjectLiteralElementLike, JsxAttribute, IfStatement, ExpressionStatement, Identifier } from 'typescript';
import { ImportCollection } from '../imports';
export declare const buildMutationBindings: (form: StudioForm) => BindingElement[];
export declare const createValidationExpression: (validationRules?: FieldValidationConfiguration[]) => Expression;
export declare const addFormAttributes: (component: StudioComponent | StudioComponentChild, formMetadata: FormMetadata) => JsxAttribute[];
export declare function buildOnBlurStatement(fieldName: string, fieldConfig: FieldConfigMetadata): JsxAttribute;
/**
 * if the onChange variable is defined it will send the current state of the fields into the function
 * the function expects all fields in return
 * the value for that fields onChange will be used from the return object for validation and updating the new state
 *
 * if a valueName override is provided it will use the provided name
 * this the name of the variable to update if onChange override function is provided
 *
 *
 * ex. if the field is email
 * const returnObject = onChange({ email, ...otherFieldsForForm });
 * const value = returnObject.email;
 *
 * this value is now used in email validation and setting the state
 */
export declare const buildOverrideOnChangeStatement: (fieldName: string, fieldConfigs: Record<string, FieldConfigMetadata>, valueNameOverride?: Identifier | undefined) => IfStatement;
export declare const buildComponentSpecificAttributes: ({ componentType }: {
    componentType: string;
}) => JsxAttribute[];
export declare const buildOnChangeStatement: (component: StudioComponent | StudioComponentChild, fieldConfigs: Record<string, FieldConfigMetadata>) => JsxAttribute;
export declare const buildDataStoreExpression: (dataStoreActionType: 'update' | 'create', importedModelName: string, dataTypeName: string) => ExpressionStatement[];
export declare const buildOverrideTypesBindings: (formComponent: StudioComponent, formDefinition: FormDefinition, importCollection: ImportCollection) => import("typescript").TypeAliasDeclaration;
/**
 * builds validation variable
 * for nested values it will mention the full path as that corresponds to the fields
 * this will also link to error messages
 *
 * const validations = { post_url: [{ type: "URL" }], 'user.status': [] };
 *
 * @param fieldConfigs
 * @returns
 */
export declare function buildValidations(fieldConfigs: Record<string, FieldConfigMetadata>): import("typescript").VariableStatement;
/**
  const runValidationTasks = async (fieldName, value) => {
    let validationResponse = validateField(value, validations[fieldName]);
    if (onValidate?.[fieldName]) {
      validationResponse = await onValidate[fieldName](value, validationResponse);
    }
    setErrors((errors) => ({ ...errors, [fieldName]: validationResponse }));
    return validationResponse;
  };
 */
export declare const runValidationTasksFunction: import("typescript").VariableStatement;
/**
 * builds modelFields object which is used to validate, onSubmit, onSuccess/onError
 * the nameOverrides will swap in a different expression instead of the name of the state when building the object
 *
 * ex.  [name, content, updatedAt]
 *
 * const modelFields = {
 *   name,
 *   content,
 *   updatedAt
 * };
 * @param fieldConfigs
 * @returns
 */
export declare const buildModelFieldObject: (shouldBeConst: boolean, fieldConfigs?: Record<string, FieldConfigMetadata>, nameOverrides?: Record<string, ObjectLiteralElementLike>) => import("typescript").VariableStatement;
/**
  const validationResponses = await Promise.all(
    Object.keys(validations).reduce((promises, fieldName) => {
        if (Array.isArray(modelFields[fieldName])) {
            promises.push(...modelFields[fieldName].map(item => runValidationTasks(fieldName, item)));
        }
        promises.push(runValidationTasks(fieldName, modelFields[fieldName]))
        return promises
    }, [])
  );

  if (validationResponses.some((r) => r.hasError)) {
    return;
  }
 */
export declare const onSubmitValidationRun: (import("typescript").VariableStatement | IfStatement)[];
export declare const buildSetStateFunction: (fieldConfigs: Record<string, FieldConfigMetadata>) => IfStatement;
export declare const buildResetValuesOnRecordUpdate: (recordName: string) => ExpressionStatement;
export declare const buildUpdateDatastoreQuery: (dataTypeName: string, recordName: string) => (ExpressionStatement | import("typescript").VariableStatement)[];
