"use strict";
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildUpdateDatastoreQuery = exports.buildResetValuesOnRecordUpdate = exports.buildSetStateFunction = exports.onSubmitValidationRun = exports.buildModelFieldObject = exports.runValidationTasksFunction = exports.buildValidations = exports.buildOverrideTypesBindings = exports.buildDataStoreExpression = exports.buildOnChangeStatement = exports.buildComponentSpecificAttributes = exports.buildOverrideOnChangeStatement = exports.buildOnBlurStatement = exports.addFormAttributes = exports.createValidationExpression = exports.buildMutationBindings = void 0;
const codegen_ui_1 = require("@aws-amplify/codegen-ui");
const typescript_1 = require("typescript");
const component_helper_1 = require("./component-helper");
const helpers_1 = require("../helpers");
const imports_1 = require("../imports");
const event_targets_1 = require("./event-targets");
const form_state_1 = require("./form-state");
const react_component_render_helper_1 = require("../react-component-render-helper");
const buildMutationBindings = (form) => {
    const { dataType: { dataSourceType, dataTypeName }, formActionType, } = form;
    const elements = [];
    if (dataSourceType === 'DataStore') {
        if (formActionType === 'update') {
            elements.push(
            // TODO: change once cpk is supported in datastore
            typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('id'), undefined), typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier((0, helpers_1.lowerCaseFirst)(dataTypeName)), undefined));
        }
        if (formActionType === 'create') {
            elements.push(typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('clearOnSuccess'), typescript_1.factory.createTrue()));
        }
        elements.push(typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('onSuccess'), undefined), typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('onError'), undefined));
    }
    if (dataSourceType === 'Custom' && formActionType === 'update') {
        elements.push(typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('initialData'), undefined));
    }
    elements.push(typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('onSubmit'), undefined));
    return elements;
};
exports.buildMutationBindings = buildMutationBindings;
const createValidationExpression = (validationRules = []) => {
    const validateExpressions = validationRules.map((rule) => {
        const elements = [
            typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier('type'), typescript_1.factory.createStringLiteral(rule.type)),
        ];
        if ('strValues' in rule) {
            elements.push(typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier('strValues'), typescript_1.factory.createArrayLiteralExpression(rule.strValues.map((value) => typescript_1.factory.createStringLiteral(value)), false)));
        }
        if ('numValues' in rule) {
            elements.push(typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier('numValues'), typescript_1.factory.createArrayLiteralExpression(rule.numValues.map((value) => typescript_1.factory.createNumericLiteral(value)), false)));
        }
        if (rule.validationMessage) {
            elements.push(typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier('validationMessage'), typescript_1.factory.createStringLiteral(rule.validationMessage)));
        }
        return typescript_1.factory.createObjectLiteralExpression(elements, false);
    });
    return typescript_1.factory.createArrayLiteralExpression(validateExpressions, true);
};
exports.createValidationExpression = createValidationExpression;
const addFormAttributes = (component, formMetadata) => {
    var _a;
    const { name: componentName, componentType } = component;
    const attributes = [];
    /*
        boolean => RadioGroupField
        const value = e.target.value.toLowerCase() === 'yes';
        boolean => selectfield
        const value = ....
    
    
        componentType => SelectField && boolean
        const value = Boolean(e.target.checked)
  
      */
    if (componentName in formMetadata.fieldConfigs) {
        const fieldConfig = formMetadata.fieldConfigs[componentName];
        const renderedVariableName = fieldConfig.sanitizedFieldName || componentName;
        /*
        if the componetName is a dotPath we need to change the access expression to the following
         - bio.user.favorites.Quote => errors['bio.user.favorites.Quote']?.errorMessage
        if it's a regular componetName it will use the following expression
         - bio => errors.bio?.errorMessage
        */
        const errorKey = componentName.split('.').length > 1 || !(0, codegen_ui_1.isValidVariableName)(componentName)
            ? typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('errors'), typescript_1.factory.createStringLiteral(componentName))
            : typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('errors'), typescript_1.factory.createIdentifier(componentName));
        attributes.push(...(0, exports.buildComponentSpecificAttributes)({
            componentType: (_a = fieldConfig.studioFormComponentType) !== null && _a !== void 0 ? _a : fieldConfig.componentType,
        }));
        const valueAttribute = (0, component_helper_1.renderValueAttribute)({
            componentName: renderedVariableName,
            currentValueIdentifier: fieldConfig.isArray ? (0, form_state_1.getCurrentValueIdentifier)(renderedVariableName) : undefined,
            fieldConfig,
        });
        if (valueAttribute) {
            attributes.push(valueAttribute);
        }
        if (formMetadata.formActionType === 'update' && !fieldConfig.isArray && !(0, component_helper_1.isControlledComponent)(componentType)) {
            attributes.push((0, component_helper_1.renderDefaultValueAttribute)(renderedVariableName, fieldConfig));
        }
        attributes.push((0, exports.buildOnChangeStatement)(component, formMetadata.fieldConfigs));
        attributes.push(buildOnBlurStatement(componentName, fieldConfig));
        attributes.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('errorMessage'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createPropertyAccessChain(errorKey, typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionDotToken), typescript_1.factory.createIdentifier('errorMessage')))), typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('hasError'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createPropertyAccessChain(errorKey, typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionDotToken), typescript_1.factory.createIdentifier('hasError')))));
        if (fieldConfig.isArray) {
            attributes.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('ref'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createIdentifier((0, form_state_1.getArrayChildRefName)(renderedVariableName)))));
        }
    }
    if (componentName === 'RightAlignCTASubFlex') {
        const flexGapAttribute = (0, react_component_render_helper_1.buildCtaLayoutProperties)(formMetadata);
        if (flexGapAttribute) {
            attributes.push(flexGapAttribute);
        }
    }
    if (componentName === 'ClearButton' || componentName === 'ResetButton') {
        attributes.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('onClick'), typescript_1.factory.createJsxExpression(undefined, form_state_1.resetValuesName)));
    }
    if (componentName === 'SubmitButton') {
        attributes.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('isDisabled'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('values')), undefined, [typescript_1.factory.createIdentifier('errors')]), typescript_1.factory.createIdentifier('some')), undefined, [
            typescript_1.factory.createArrowFunction(undefined, undefined, [
                typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('e'), undefined, undefined, undefined),
            ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createPropertyAccessChain(typescript_1.factory.createIdentifier('e'), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionDotToken), typescript_1.factory.createIdentifier('hasError'))),
        ]))));
    }
    if (componentName === 'CancelButton') {
        attributes.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('onClick'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createArrowFunction(undefined, undefined, [], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('onCancel'), typescript_1.factory.createToken(typescript_1.SyntaxKind.AmpersandAmpersandToken), typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('onCancel'), undefined, []))),
        ], false)))));
    }
    return attributes;
};
exports.addFormAttributes = addFormAttributes;
/**
  if (errors.name?.hasError) {
    runValidationTasks("name", value);
  }
 */
function getOnChangeValidationBlock(fieldName) {
    return typescript_1.factory.createIfStatement(typescript_1.factory.createPropertyAccessChain((0, codegen_ui_1.isValidVariableName)(fieldName)
        ? typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('errors'), typescript_1.factory.createIdentifier(fieldName))
        : typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('errors'), typescript_1.factory.createStringLiteral(fieldName)), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionDotToken), typescript_1.factory.createIdentifier('hasError')), typescript_1.factory.createBlock([
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('runValidationTasks'), undefined, [
            typescript_1.factory.createStringLiteral(fieldName),
            typescript_1.factory.createIdentifier('value'),
        ])),
    ], true), undefined);
}
function buildOnBlurStatement(fieldName, fieldConfig) {
    const renderedFieldName = fieldConfig.sanitizedFieldName || fieldName;
    let fieldNameIdentifier = typescript_1.factory.createIdentifier(renderedFieldName);
    if (fieldName.includes('.')) {
        const [parent, child] = fieldName.split('.');
        fieldNameIdentifier = typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier(parent), typescript_1.factory.createStringLiteral(child));
    }
    return typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('onBlur'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createArrowFunction(undefined, undefined, [], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('runValidationTasks'), undefined, [
        typescript_1.factory.createStringLiteral(fieldName),
        fieldConfig.isArray ? (0, form_state_1.getCurrentValueIdentifier)(renderedFieldName) : fieldNameIdentifier,
    ]))));
}
exports.buildOnBlurStatement = buildOnBlurStatement;
/**
 * if the onChange variable is defined it will send the current state of the fields into the function
 * the function expects all fields in return
 * the value for that fields onChange will be used from the return object for validation and updating the new state
 *
 * if a valueName override is provided it will use the provided name
 * this the name of the variable to update if onChange override function is provided
 *
 *
 * ex. if the field is email
 * const returnObject = onChange({ email, ...otherFieldsForForm });
 * const value = returnObject.email;
 *
 * this value is now used in email validation and setting the state
 */
const buildOverrideOnChangeStatement = (fieldName, fieldConfigs, valueNameOverride) => {
    const keyPath = fieldName.split('.');
    const keyName = keyPath[0];
    const valueName = valueNameOverride !== null && valueNameOverride !== void 0 ? valueNameOverride : typescript_1.factory.createIdentifier('value');
    let keyValueExpression = typescript_1.factory.createPropertyAssignment((0, codegen_ui_1.isValidVariableName)(keyName) ? typescript_1.factory.createIdentifier(keyName) : typescript_1.factory.createStringLiteral(keyName), valueName);
    if (keyPath.length > 1) {
        keyValueExpression = typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(keyName), (0, form_state_1.buildNestedStateSet)(keyPath, [keyName], valueName));
    }
    return typescript_1.factory.createIfStatement(typescript_1.factory.createIdentifier('onChange'), typescript_1.factory.createBlock([
        (0, exports.buildModelFieldObject)(true, fieldConfigs, {
            [keyName]: keyValueExpression,
        }),
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('result'), undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('onChange'), undefined, [
                typescript_1.factory.createIdentifier('modelFields'),
            ])),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createBinaryExpression(valueName, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsToken), typescript_1.factory.createBinaryExpression((0, form_state_1.buildAccessChain)(['result', ...fieldName.split('.')]), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionQuestionToken), valueName))),
    ], true), undefined);
};
exports.buildOverrideOnChangeStatement = buildOverrideOnChangeStatement;
function getOnValueChangeProp(fieldType) {
    var _a;
    const map = {
        StepperField: 'onStepChange',
    };
    return (_a = map[fieldType]) !== null && _a !== void 0 ? _a : 'onChange';
}
const buildComponentSpecificAttributes = ({ componentType }) => {
    var _a;
    const componentToAttributesMap = {
        NumberField: [typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('step'), typescript_1.factory.createStringLiteral('any'))],
    };
    return (_a = componentToAttributesMap[componentType]) !== null && _a !== void 0 ? _a : [];
};
exports.buildComponentSpecificAttributes = buildComponentSpecificAttributes;
const buildOnChangeStatement = (component, fieldConfigs) => {
    const { name: fieldName, componentType: fieldType } = component;
    const { dataType, isArray, sanitizedFieldName } = fieldConfigs[fieldName];
    const renderedFieldName = sanitizedFieldName || fieldName;
    if (isArray) {
        return typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier(getOnValueChangeProp(fieldType)), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createArrowFunction(undefined, undefined, [
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('e'), undefined, undefined, undefined),
        ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
            ...(0, event_targets_1.buildTargetVariable)(fieldType, fieldName, dataType),
            getOnChangeValidationBlock(fieldName),
            (0, form_state_1.setStateExpression)((0, form_state_1.getCurrentValueName)(renderedFieldName), (0, event_targets_1.getFormattedValueExpression)(dataType)),
        ], true))));
    }
    return typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier(getOnValueChangeProp(fieldType)), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createArrowFunction(undefined, undefined, [
        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('e'), undefined, undefined, undefined),
    ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
        ...(0, event_targets_1.buildTargetVariable)(fieldType, fieldName, dataType),
        (0, exports.buildOverrideOnChangeStatement)(fieldName, fieldConfigs),
        getOnChangeValidationBlock(fieldName),
        typescript_1.factory.createExpressionStatement((0, form_state_1.setFieldState)(renderedFieldName, (0, event_targets_1.getFormattedValueExpression)(dataType))),
    ], true))));
};
exports.buildOnChangeStatement = buildOnChangeStatement;
const buildDataStoreExpression = (dataStoreActionType, importedModelName, dataTypeName) => {
    if (dataStoreActionType === 'update') {
        return [
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(importedModelName), typescript_1.factory.createIdentifier('copyOf')), undefined, [
                    typescript_1.factory.createIdentifier(`${(0, helpers_1.lowerCaseFirst)(dataTypeName)}Record`),
                    typescript_1.factory.createArrowFunction(undefined, undefined, [
                        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('updated'), undefined, undefined, undefined),
                    ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('assign')), undefined, [typescript_1.factory.createIdentifier('updated'), typescript_1.factory.createIdentifier('modelFields')])),
                    ], true)),
                ]),
            ]))),
        ];
    }
    return [
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
            typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier(importedModelName), undefined, [
                typescript_1.factory.createIdentifier('modelFields'),
            ]),
        ]))),
    ];
};
exports.buildDataStoreExpression = buildDataStoreExpression;
const buildOverrideTypesBindings = (formComponent, formDefinition, importCollection) => {
    importCollection.addImport(imports_1.ImportSource.UI_REACT, 'GridProps');
    const typeNodes = [
        typescript_1.factory.createPropertySignature(undefined, typescript_1.factory.createIdentifier(`${formComponent.name}Grid`), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('FormProps'), [
            typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('GridProps'), undefined),
        ])),
    ];
    formDefinition.elementMatrix.forEach((row, index) => {
        if (row.length > 1) {
            typeNodes.push(typescript_1.factory.createPropertySignature(undefined, typescript_1.factory.createIdentifier(`RowGrid${index}`), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('FormProps'), [
                typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('GridProps'), undefined),
            ])));
        }
        row.forEach((field) => {
            let propKey = typescript_1.factory.createIdentifier(field);
            if (field.split('.').length > 1 || !(0, codegen_ui_1.isValidVariableName)(field)) {
                propKey = typescript_1.factory.createStringLiteral(field);
            }
            const componentTypePropName = `${formDefinition.elements[field].componentType}Props`;
            typeNodes.push(typescript_1.factory.createPropertySignature(undefined, propKey, typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('FormProps'), [
                typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier(componentTypePropName), undefined),
            ])));
            importCollection.addImport(imports_1.ImportSource.UI_REACT, componentTypePropName);
        });
    });
    return typescript_1.factory.createTypeAliasDeclaration(undefined, [typescript_1.factory.createModifier(typescript_1.SyntaxKind.ExportKeyword), typescript_1.factory.createModifier(typescript_1.SyntaxKind.DeclareKeyword)], typescript_1.factory.createIdentifier(`${formComponent.name}OverridesProps`), undefined, typescript_1.factory.createIntersectionTypeNode([
        typescript_1.factory.createTypeLiteralNode(typeNodes),
        typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('EscapeHatchProps'), undefined),
    ]));
};
exports.buildOverrideTypesBindings = buildOverrideTypesBindings;
/**
 * builds validation variable
 * for nested values it will mention the full path as that corresponds to the fields
 * this will also link to error messages
 *
 * const validations = { post_url: [{ type: "URL" }], 'user.status': [] };
 *
 * @param fieldConfigs
 * @returns
 */
function buildValidations(fieldConfigs) {
    const validationsForField = Object.entries(fieldConfigs).map(([fieldName, { validationRules }]) => {
        const propKey = fieldName.split('.').length > 1 || !(0, codegen_ui_1.isValidVariableName)(fieldName)
            ? typescript_1.factory.createStringLiteral(fieldName)
            : typescript_1.factory.createIdentifier(fieldName);
        return typescript_1.factory.createPropertyAssignment(propKey, (0, exports.createValidationExpression)(validationRules));
    });
    return typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('validations'), undefined, undefined, typescript_1.factory.createObjectLiteralExpression(validationsForField, true)),
    ], typescript_1.NodeFlags.Const));
}
exports.buildValidations = buildValidations;
/**
  const runValidationTasks = async (fieldName, value) => {
    let validationResponse = validateField(value, validations[fieldName]);
    if (onValidate?.[fieldName]) {
      validationResponse = await onValidate[fieldName](value, validationResponse);
    }
    setErrors((errors) => ({ ...errors, [fieldName]: validationResponse }));
    return validationResponse;
  };
 */
exports.runValidationTasksFunction = typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
    typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('runValidationTasks'), undefined, undefined, typescript_1.factory.createArrowFunction([typescript_1.factory.createModifier(typescript_1.SyntaxKind.AsyncKeyword)], undefined, [
        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('fieldName'), undefined, undefined, undefined),
        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('value'), undefined, undefined, undefined),
    ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('validationResponse'), undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('validateField'), undefined, [
                typescript_1.factory.createIdentifier('value'),
                typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('validations'), typescript_1.factory.createIdentifier('fieldName')),
            ])),
        ], typescript_1.NodeFlags.Let)),
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('customValidator'), undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('fetchByPath'), undefined, [
                typescript_1.factory.createIdentifier('onValidate'),
                typescript_1.factory.createIdentifier('fieldName'),
            ])),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createIfStatement(typescript_1.factory.createIdentifier('customValidator'), typescript_1.factory.createBlock([
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier('validationResponse'), typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsToken), typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('customValidator'), undefined, [
                typescript_1.factory.createIdentifier('value'),
                typescript_1.factory.createIdentifier('validationResponse'),
            ])))),
        ], true), undefined),
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('setErrors'), undefined, [
            typescript_1.factory.createArrowFunction(undefined, undefined, [
                typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('errors'), undefined, undefined, undefined),
            ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createParenthesizedExpression(typescript_1.factory.createObjectLiteralExpression([
                typescript_1.factory.createSpreadAssignment(typescript_1.factory.createIdentifier('errors')),
                typescript_1.factory.createPropertyAssignment(typescript_1.factory.createComputedPropertyName(typescript_1.factory.createIdentifier('fieldName')), typescript_1.factory.createIdentifier('validationResponse')),
            ], false))),
        ])),
        typescript_1.factory.createReturnStatement(typescript_1.factory.createIdentifier('validationResponse')),
    ], true))),
], typescript_1.NodeFlags.Const));
/**
 * builds modelFields object which is used to validate, onSubmit, onSuccess/onError
 * the nameOverrides will swap in a different expression instead of the name of the state when building the object
 *
 * ex.  [name, content, updatedAt]
 *
 * const modelFields = {
 *   name,
 *   content,
 *   updatedAt
 * };
 * @param fieldConfigs
 * @returns
 */
const buildModelFieldObject = (shouldBeConst, fieldConfigs = {}, nameOverrides = {}) => {
    const fieldSet = new Set();
    const fields = Object.keys(fieldConfigs).reduce((acc, value) => {
        const fieldName = value.split('.')[0];
        const { sanitizedFieldName, dataType } = fieldConfigs[value];
        const renderedFieldName = sanitizedFieldName || fieldName;
        if (!fieldSet.has(renderedFieldName)) {
            let assignment = typescript_1.factory.createShorthandPropertyAssignment(typescript_1.factory.createIdentifier(fieldName), undefined);
            if (nameOverrides[fieldName]) {
                assignment = nameOverrides[fieldName];
            }
            else if (sanitizedFieldName) {
                // if overrides present, ignore sanitizedFieldName
                assignment = typescript_1.factory.createPropertyAssignment(typescript_1.factory.createStringLiteral(fieldName), typescript_1.factory.createIdentifier(sanitizedFieldName));
            }
            /*
             Empty string value for not required url field fails to save at datastore
             let modelFields = {
              url: url || undefined,
             }
            */
            if (dataType === 'AWSURL' && !shouldBeConst) {
                assignment = typescript_1.factory.createPropertyAssignment(typescript_1.factory.createStringLiteral(fieldName), typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier(renderedFieldName), typescript_1.factory.createToken(typescript_1.SyntaxKind.BarBarToken), typescript_1.factory.createIdentifier('undefined')));
            }
            acc.push(assignment);
            fieldSet.add(renderedFieldName);
        }
        return acc;
    }, []);
    return typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('modelFields'), undefined, undefined, typescript_1.factory.createObjectLiteralExpression(fields, true)),
    ], shouldBeConst ? typescript_1.NodeFlags.Const : typescript_1.NodeFlags.Let));
};
exports.buildModelFieldObject = buildModelFieldObject;
/**
  const validationResponses = await Promise.all(
    Object.keys(validations).reduce((promises, fieldName) => {
        if (Array.isArray(modelFields[fieldName])) {
            promises.push(...modelFields[fieldName].map(item => runValidationTasks(fieldName, item)));
        }
        promises.push(runValidationTasks(fieldName, modelFields[fieldName]))
        return promises
    }, [])
  );

  if (validationResponses.some((r) => r.hasError)) {
    return;
  }
 */
exports.onSubmitValidationRun = [
    typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('validationResponses'), undefined, undefined, typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Promise'), typescript_1.factory.createIdentifier('all')), undefined, [
            typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('keys')), undefined, [typescript_1.factory.createIdentifier('validations')]), typescript_1.factory.createIdentifier('reduce')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('promises'), undefined, undefined),
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('fieldName'), undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createIfStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Array'), typescript_1.factory.createIdentifier('isArray')), undefined, [
                        typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier('fieldName')),
                    ]), typescript_1.factory.createBlock([
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                            typescript_1.factory.createSpreadElement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier('fieldName')), typescript_1.factory.createIdentifier('map')), undefined, [
                                typescript_1.factory.createArrowFunction(undefined, undefined, [
                                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('item'), undefined, undefined),
                                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('runValidationTasks'), undefined, [
                                    typescript_1.factory.createIdentifier('fieldName'),
                                    typescript_1.factory.createIdentifier('item'),
                                ])),
                            ])),
                        ])),
                        typescript_1.factory.createReturnStatement(typescript_1.factory.createIdentifier('promises')),
                    ], true), undefined),
                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                        typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('runValidationTasks'), undefined, [
                            typescript_1.factory.createIdentifier('fieldName'),
                            typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier('fieldName')),
                        ]),
                    ])),
                    typescript_1.factory.createReturnStatement(typescript_1.factory.createIdentifier('promises')),
                ], true)),
                typescript_1.factory.createArrayLiteralExpression([], false),
            ]),
        ]))),
    ], typescript_1.NodeFlags.Const)),
    typescript_1.factory.createIfStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('validationResponses'), typescript_1.factory.createIdentifier('some')), undefined, [
        typescript_1.factory.createArrowFunction(undefined, undefined, [
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('r'), undefined, undefined, undefined),
        ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('hasError'))),
    ]), typescript_1.factory.createBlock([typescript_1.factory.createReturnStatement(undefined)], true), undefined),
];
const buildSetStateFunction = (fieldConfigs) => {
    const fieldSet = new Set();
    const expression = Object.keys(fieldConfigs).reduce((acc, field) => {
        const fieldName = field.split('.')[0];
        const renderedFieldName = fieldConfigs[field].sanitizedFieldName || fieldName;
        if (!fieldSet.has(renderedFieldName)) {
            acc.push(typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier(`set${(0, form_state_1.capitalizeFirstLetter)(renderedFieldName)}`), undefined, [
                (0, codegen_ui_1.isValidVariableName)(fieldName)
                    ? typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('initialData'), typescript_1.factory.createIdentifier(fieldName))
                    : typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('initialData'), typescript_1.factory.createStringLiteral(fieldName)),
            ])));
            fieldSet.add(renderedFieldName);
        }
        return acc;
    }, []);
    return typescript_1.factory.createIfStatement(typescript_1.factory.createIdentifier('initialData'), typescript_1.factory.createBlock(expression, true));
};
exports.buildSetStateFunction = buildSetStateFunction;
// ex. React.useEffect(resetStateValues, [bookRecord])
const buildResetValuesOnRecordUpdate = (recordName) => {
    return typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('React'), typescript_1.factory.createIdentifier('useEffect')), undefined, [form_state_1.resetValuesName, typescript_1.factory.createArrayLiteralExpression([typescript_1.factory.createIdentifier(recordName)], false)]));
};
exports.buildResetValuesOnRecordUpdate = buildResetValuesOnRecordUpdate;
const buildUpdateDatastoreQuery = (dataTypeName, recordName) => {
    // TODO: update this once cpk is supported in datastore
    const pkQueryIdentifier = typescript_1.factory.createIdentifier('id');
    return [
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('queryData'), undefined, undefined, typescript_1.factory.createArrowFunction([typescript_1.factory.createModifier(typescript_1.SyntaxKind.AsyncKeyword)], undefined, [], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                    typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('record'), undefined, undefined, typescript_1.factory.createConditionalExpression(pkQueryIdentifier, typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('query')), undefined, [typescript_1.factory.createIdentifier(dataTypeName), pkQueryIdentifier])), typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createIdentifier((0, helpers_1.lowerCaseFirst)(dataTypeName)))),
                ], typescript_1.NodeFlags.Const)),
                typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression((0, form_state_1.getSetNameIdentifier)(recordName), undefined, [
                    typescript_1.factory.createIdentifier('record'),
                ])),
            ], true))),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('queryData'), undefined, [])),
    ];
};
exports.buildUpdateDatastoreQuery = buildUpdateDatastoreQuery;
//# sourceMappingURL=form-renderer-helper.js.map